﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using NToastNotify;
using WebPortal.Models;
using WebPortal.ViewModels;

namespace WebPortal.Controllers
{
    public class HomeController : Controller
    {
        public List<Visite> listaVisite;                

        public async Task<IActionResult> EditDetailsVisit(string index)
        {
            try
            {
                HttpClient client = new HttpClient();
                HttpRequestMessage request = new HttpRequestMessage(HttpMethod.Get, "https://localhost/api/Visite/GetVisita/" + index);

                Visite v = null;
                HttpContext.Session.TryGetValue("token", out byte[] token);
                if (token == null) return RedirectToAction("Index", "Login");
                string t = Encoding.UTF8.GetString(token);
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", t);

                client.SetBearerToken(t);
                var resp = await client.SendAsync(request);

                if (resp.IsSuccessStatusCode)
                {
                    string res = await resp.Content.ReadAsStringAsync();
                    v = JsonConvert.DeserializeObject<Visite>(res);

                    if (v.Modulo == null)
                    {
                        v.Modulo = new ModuloVisita();
                        v.Modulo.IdVisita = v.Id;                        
                    }

                    if (v.ListaFoto == null) v.ListaFoto = new List<Foto>();
                    if (v.Operazioni == null) v.Operazioni = new List<OperazioniVisita>();

                    InsertNewVisitViewModel vm = await GetNewVisitViewModel(t);

                    if (vm != null)
                    {
                        if (string.IsNullOrWhiteSpace(v.Area))
                        {
                            List<Area> aree = new List<Area>();

                            aree.Add(new Area() { Codice = "", Descrizione = "" });

                            if (vm.Aree != null) aree.AddRange(vm.Aree);

                            vm.Aree = aree;
                        }
                        
                        if (v.Modulo.IdFunzionario == 0)
                        {
                            List<Funzionari> funzionari = new List<Funzionari>();

                            funzionari.Add(new Funzionari() { Id = 0, Ente = "", Nominativo = "" });

                            if (vm.Funzionari != null) funzionari.AddRange(vm.Funzionari);

                            vm.Funzionari = funzionari;
                        }
                        
                        if (string.IsNullOrWhiteSpace(v.Modulo.NomeChecker))
                        {
                            List<Checker> checkers = new List<Checker>();

                            checkers.Add(new Checker() { Id = 0, Ente = "", Nominativo = "" });

                            if (vm.Checkers != null) checkers.AddRange(vm.Checkers);

                            vm.Checkers = checkers;
                        }                       

                        ViewBag.NewVisitViewModel = vm;

                        return View(v);
                    }
                    else return Error();
                }
                else
                    return Error();                
            }
            catch (Exception) { return Error(); }
        }

        public async Task<IActionResult> SaveDetailsVisit(Visite visita, string save)
        {
            try
            {
                if (!ModelState.IsValid || visita == null || visita.Id == 0 || string.IsNullOrWhiteSpace(visita.VisitId) || string.IsNullOrEmpty(save))
                {
                    if (string.IsNullOrEmpty(save))
                        return null;
                    else
                        return RedirectToAction("Index", "Home");                    
                }
                else
                {
                    //Salva su server                    
                    string d = Request.Form["DataProgrammataValue"];                    

                    if (!string.IsNullOrWhiteSpace(d))
                    {
                        visita.DataProgrammata = Convert.ToDateTime(d);
                    }
                    else if (visita.DataProgrammata == null)
                    {
                        visita.DataProgrammata = DateTime.Now;
                    }

                    visita.TimestampRegistrazione = DateTimeOffset.Now;
                    visita.TimestampUltimaModifica = DateTimeOffset.Now;

                    HttpContext.Session.TryGetValue("username", out byte[] user);
                    visita.UsernameRegistrazione = Encoding.UTF8.GetString(user);
                    visita.UsernameUltimaModifica = visita.UsernameRegistrazione;

                    HttpContext.Session.TryGetValue("token", out byte[] token);

                    if (token == null) return RedirectToAction("Index", "Login");

                    HttpClient client = new HttpClient();

                    StringContent content = new StringContent(JsonConvert.SerializeObject(visita), Encoding.UTF8, "application/json");

                    string t = Encoding.UTF8.GetString(token);
                    client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", t);

                    client.SetBearerToken(t);                    
                    var resp = await client.PutAsync("https://localhost:6001/api/Visite/UpdateVisita/" + visita.Id, content);

                    if (resp.IsSuccessStatusCode)
                    {
                        ToastNotification.AddSuccessToastMessage("Modifiche salvate");
                        
                        return RedirectToAction("ViewDetailsVisit", "Home", new { index = visita.VisitId });
                    }
                    else
                    {
                        string res = await resp.Content.ReadAsStringAsync();
                        ToastNotification.AddErrorToastMessage($"Errore inserimento: {res}");

                        return RedirectToAction("EditDetailsVisit", "Home", new { index = visita.VisitId });
                    }
                }
            }
            catch (Exception ex) { var ee = ex; return null; }
        }        

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}